## Source some functions
source("nllikelihood.R")

## Read some test data
X <- read.csv("inputPRBS1.csv",sep=";",header=TRUE)

##
require(ctsmr)
## Generate a new object of class ctsm
model <- ctsm()
## Add a system equation and thereby also a state
model$addSystem(dTi ~ ( 1/(Ci*Ria)*(Ta-Ti) + Aw/Ci*Ps + 1/Ci*Ph )*dt + exp(p11)*dw1)
## Set the names of the inputs
model$addInput(Ta,Ps,Ph)
## Set the observation equation: Ti is the state, yTi is the measured output
model$addObs(yTi ~ Ti)
## Set the variance of the measurement error
model$setVariance(yTi ~ exp(e11))
## Set the initial value (for the optimization) of the value of the state at the starting time point
model$setParameter(  Ti = c(init=15  ,lb=0     ,ub=25 ) )
## Set the initial value for the optimization
model$setParameter(  Ci = c(init=1   ,lb=1E-5  ,ub=20 ) )
model$setParameter( Ria = c(init=20  ,lb=1     ,ub=1E4) )
model$setParameter(  Aw = c(init=20  ,lb=1     ,ub=300) )
model$setParameter( p11 = c(init=1   ,lb=-30   ,ub=10 ) )
model$setParameter( e11 = c(init=-1  ,lb=-50   ,ub=10 ) )

## Needs to be runned for simulate and predict to work
model$AnalyseModel()

## Make a fit object to pass to simulate.ctsmr
fit <- list()
fit$model <- model
fit$xm <- model$ParameterValues$initial
names(fit$xm) <- row.names(model$ParameterValues)
## Can be nessary such that xm has the right length
fit$xm <- fit$xm[model$pars]
class(fit) <- "ctsmr"

## The likelihood of the initial parameters set in fit
nllikelihood(fit$xm[model$pars], fit, D=X, firstorder=TRUE, c=3, n.ahead=1)



## ------------------------------------------------------------------
## Fit with different optimizers

## Fit with ctsmr
fitctsmr <- model$estimate(X, firstorder=TRUE)

## Fit with nlminb
fitnlminb <- nlminb(fit$xm[model$pars], nllikelihood, fit=fit, D=X, firstorder=TRUE, c=3, n.ahead=1)

## Fit with optim
fitoptim <- optim(fit$xm[model$pars], nllikelihood, fit=fit, D=X, firstorder=TRUE, c=3, n.ahead=1)

## Fit with optim BFGS
fitoptimBFGS <- optim(fit$xm[model$pars], nllikelihood, fit=fit, D=X, firstorder=TRUE, c=3, n.ahead=1, method="L-BFGS-B", lower=model$ParameterValues$lower, upper=model$ParameterValues$upper)
## ------------------------------------------------------------------


## ------------------------------------------------------------------
## Compare the results

## First check the nllikelihood function
-(fitctsmr$f - fitctsmr$fpen)
-nllikelihood(fitctsmr$xm, fitctsmr, D=X, firstorder=TRUE, c=3, n.ahead=1)

## Then check the max likelihoods
-nllikelihood(fitctsmr$xm, fitctsmr, D=X, firstorder=TRUE, c=3, n.ahead=1)
-nllikelihood(fitnlminb$par, fit, D=X, firstorder=TRUE, c=3, n.ahead=1)
## Same max likelihood

## These two are not to good
-nllikelihood(fitoptim$par, fit, D=X, firstorder=TRUE, c=3, n.ahead=1)
-nllikelihood(fitoptimBFGS$par, fit, D=X, firstorder=TRUE, c=3, n.ahead=1)


## Check the parameters
fitctsmr$xm
fitnlminb$par
fitoptim$par
fitoptimBFGS$par


## Plot the simulated output from the three estimated models
insert_xm <- function(fit, xm){
    fit$xm <- xm
    names(fit$xm) <- names(xm)
    fit$xm <- fit$xm[fit$model$pars]
    return(fit)
}

## ctsmr
(fit <- insert_xm(fit, fitctsmr$xm))
sim <- simulate(fit, newdata=X)
plot(X$t, sim$output$sim$yTi, type='l', ylim=range(sim$output$sim$yTi, X$yTi))
## nlminb
(fit <- insert_xm(fit, fitnlminb$par))
sim <- simulate(fit, newdata=X)
lines(X$t, sim$output$sim$yTi, col=3)
## optim
(fit <- insert_xm(fit, fitoptim$par))
sim <- simulate(fit, newdata=X)
lines(X$t, sim$output$sim$yTi, col=2)

## Conclusion is that nlminb works excellent, at least for this particular example!
## ------------------------------------------------------------------


## ------------------------------------------------------------------
## Estimate the uncertainty of the parameters from nlminb fit
(fit <- insert_xm(fit, fitnlminb$par))

## Use the hessian function function from the numDeriv package
require(numDeriv)

## Approximate the Hessian
H <- hessian(nllikelihood, x=fit$xm, fit=fit, D=X, firstorder=TRUE, c=3, n.ahead=1)

## Fisher's information matrix
I <- qr.solve(H)
colnames(I) <- names(fit$xm)

## Covariance matrix of the estimates
cov2cor(I)

## Standard error (Standard deviation estimate of the parameters)
stderror <- sqrt(diag(I))
names(stderror) <- names(fit$xm)
stderror
## t-value and t-test p-value
tval <- fit$xm/stderror
pval <- 1 - qnorm(abs(tval))
pval <- ifelse(is.nan(pval), "<e-16", format(pval,digits=4))
## Generate summary table
format(data.frame(fit$xm, stderror, tval, pval), digits=2, nsmall=4, scientific=FALSE)

## Compare with ctsmr estimates
summary(fitctsmr)


## Confidence intervals with significance level alpha
alpha <- 0.05
CI <- t(sapply(1:length(stderror), function(i){
    c(names(stderror)[i], format(fit$xm[i] + c(-1,1) * qnorm(1-alpha/2) * stderror[i], digits=4))
}))
colnames(CI) <- c("prm","lower","upper")
CI
## ------------------------------------------------------------------
